import { Button } from '@/lib/components/buttons';
import { DropdownSelector } from '@/lib/components/DropdownSelector/DropdownSelector';
import { Input } from '@/lib/components/inputs';
import { EntityPill } from '@/lib/components/pills/Pill';
import { SettingsContainer } from '@/lib/vis/components/config';
import React, { forwardRef, RefObject, useEffect, useImperativeHandle, useMemo, useRef, useState } from 'react';
import { VISComponentType, VisualizationPropTypes, VisualizationSettingsPropTypes } from '../../common';
import { exportImageInternal } from './components/utils';
import { getAttributeValues, getAttributeValuesSimple, vis1DHistogramAggregationOptions, Vis1DProps, Vis1DVisHandle } from './model';
import { Vis1DSettingsLine, Vis1DLine } from './subPlots/Vis1DLine';
import { Vis1DSettingsScatter, Vis1DScatter } from './subPlots/Vis1DScatter';
import { Vis1DPie } from './subPlots/Vis1DPie';
import { Vis1DSettingsBar, Vis1DBar } from './subPlots/Vis1DBar';
import { Vis1DSettingsHistogramCount, Vis1DHistogramCount } from './subPlots/Vis1DHistogramCount';

import { plotTypeOptions } from './subPlots/types';

const defaultSettings: Vis1DProps = {
  plotType: 'line',
  title: '',
  selectedEntity: '',
  xAxisLabel: { name: '', type: '' },
  yAxisLabel: { name: '', type: '' },
  zAxisLabel: { name: '', type: '' },
  showAxis: true,
  groupDataByTime: undefined,
  stack: false,
};

const Vis1D = forwardRef<Vis1DVisHandle, VisualizationPropTypes<Vis1DProps>>(({ data, settings }, refExternal) => {
  const internalRef = useRef<HTMLDivElement>(null);

  useImperativeHandle(refExternal, () => ({
    exportImageInternal: () => {
      if (internalRef.current) {
        exportImageInternal(internalRef as RefObject<HTMLDivElement>);
      } else {
        console.error('internalRef is null');
      }
    },
  }));

  const xAxisData = useMemo(
    () => getAttributeValuesSimple(data, settings.selectedEntity, settings.xAxisLabel),
    [data, settings.selectedEntity, settings.xAxisLabel],
  );

  const yAxisData = useMemo(
    () => getAttributeValuesSimple(data, settings.selectedEntity, settings.yAxisLabel),
    [data, settings.selectedEntity, settings.yAxisLabel],
  );

  const zAxisData = useMemo(
    () => getAttributeValuesSimple(data, settings.selectedEntity, settings.zAxisLabel),
    [data, settings.selectedEntity, settings.zAxisLabel],
  );

  return (
    <div className="h-full w-full flex items-center justify-center overflow-hidden" ref={internalRef}>
      {settings.plotType === 'line' && (
        <Vis1DLine xData={xAxisData} yData={yAxisData} showAxis={settings.showAxis} groupDataByTime={settings.groupDataByTime} />
      )}
      {settings.plotType === 'scatter' && (
        <Vis1DScatter xData={xAxisData} yData={yAxisData} zData={zAxisData} showAxis={settings.showAxis} />
      )}
      {settings.plotType === 'pie' && <Vis1DPie xData={xAxisData} showAxis={settings.showAxis} />}
      {settings.plotType === 'bar' && (
        <Vis1DBar
          xData={xAxisData}
          yData={yAxisData}
          zData={zAxisData}
          showAxis={settings.showAxis}
          groupDataByTime={settings.groupDataByTime}
        />
      )}
      {settings.plotType === 'histogram' && (
        <Vis1DHistogramCount
          xData={xAxisData}
          yData={yAxisData}
          zData={zAxisData}
          showAxis={settings.showAxis}
          groupByTime={settings.groupDataByTime}
          stackVariable={settings.stack}
        />
      )}
    </div>
  );
});

const Vis1DSettings = ({ settings, graphMetadata, updateSettings }: VisualizationSettingsPropTypes<Vis1DProps>) => {
  const mutablePlotTypes = [...plotTypeOptions];
  const [attributeOptions, setAttributeOptions] = useState<{ name: string; type: string }[]>([]);
  const [filterXLabel, setFilterXLabel] = useState<string>('');
  const [filterYLabel, setFilterYLabel] = useState<string>('');
  const [filterZLabel, setFilterZLabel] = useState<string>('');

  useEffect(() => {
    if (graphMetadata && graphMetadata.nodes && graphMetadata.nodes.labels.length > 0) {
      if (settings.selectedEntity === '') {
        updateSettings({ selectedEntity: graphMetadata.nodes.labels[0] });
      } else {
        const labelNodes = graphMetadata.nodes.labels;
        if (!labelNodes.includes(settings.selectedEntity)) {
          updateSettings({ selectedEntity: graphMetadata.nodes.labels[0] });
        }
      }
    }
  }, [graphMetadata]);

  useEffect(() => {
    if (graphMetadata && graphMetadata.nodes && graphMetadata.nodes.labels.length > 0 && settings.selectedEntity != '') {
      const labelNodes = graphMetadata.nodes.labels;
      if (labelNodes.includes(settings.selectedEntity)) {
        const newAttributeOptions: { name: string; type: string }[] = [
          { name: 'No selection', type: 'cross' },
          ...Object.entries(graphMetadata.nodes.types[settings.selectedEntity].attributes)
            .filter(([key]) => key.toLowerCase().includes(filterXLabel.toLowerCase()))
            .map(([key, value]) => ({
              name: key,
              type: value.attributeType,
            })),
        ];

        if (settings.xAxisLabel.name === '') {
          updateSettings({ xAxisLabel: { name: newAttributeOptions[0].name, type: newAttributeOptions[0].type } });

          // !TODO: instead of contain "datum" chekc type: if it is date
          if (newAttributeOptions[0].name.includes('Datum')) {
            updateSettings({ groupDataByTime: 'yearly' });
          } else {
            updateSettings({ groupDataByTime: undefined });
          }
        }
        setAttributeOptions(newAttributeOptions);
      }
    }
  }, [graphMetadata, settings.selectedEntity, filterXLabel]);

  return (
    <SettingsContainer>
      <div className="p-1">
        <Input
          className="w-full text-justify justify-start mb-2"
          type="dropdown"
          label=""
          value={settings.selectedEntity}
          options={graphMetadata.nodes.labels}
          onChange={val => updateSettings({ selectedEntity: val as string })}
          overrideRender={
            <EntityPill
              title={
                <div className="flex flex-row justify-between items-center cursor-pointer">
                  <span>{settings.selectedEntity || ''}</span>
                  <Button variantType="secondary" variant="ghost" size="2xs" iconComponent="icon-[ic--baseline-arrow-drop-down]" />
                </div>
              }
            />
          }
        />
        <div className="mb-2">
          <Input type="text" label="Title" value={settings.title} onChange={value => updateSettings({ title: value as string })} />
        </div>
        <div className="mb-2">
          <Input
            type="dropdown"
            label="Chart"
            value={settings.plotType as string}
            options={mutablePlotTypes}
            onChange={value => {
              updateSettings({ plotType: value as (typeof plotTypeOptions)[number] });
              {
                if (value === 'histogram') {
                  updateSettings({ yAxisLabel: { name: '', type: '' }, groupDataByTime: 'count' });
                } else if (value === 'bar' || value === 'pie') {
                  updateSettings({ yAxisLabel: { name: '', type: '' }, groupDataByTime: undefined });
                }
              }
            }}
          />
        </div>
        <div className="mb-2">
          <DropdownSelector
            label="X Axis"
            selectedValue={settings.xAxisLabel.name}
            options={attributeOptions}
            filter={filterXLabel}
            setFilter={setFilterXLabel}
            onChange={selected => {
              updateSettings({ xAxisLabel: { name: selected.name, type: selected.type } });

              if (!selected.name.includes('Datum') || selected.type !== 'Date') {
                updateSettings({ groupDataByTime: undefined });
              } else {
                updateSettings({ groupDataByTime: 'monthly' });
              }
            }}
          />
        </div>

        {settings.plotType === 'line' && (
          <Vis1DSettingsLine
            settings={settings}
            updateSettings={updateSettings}
            attributeOptions={attributeOptions}
            filterYLabel={filterYLabel}
            setFilterYLabel={setFilterYLabel}
          />
        )}
        {settings.plotType === 'scatter' && (
          <Vis1DSettingsScatter
            settings={settings}
            updateSettings={updateSettings}
            attributeOptions={attributeOptions}
            filterYLabel={filterYLabel}
            setFilterYLabel={setFilterYLabel}
            filterZLabel={filterZLabel}
            setFilterZLabel={setFilterZLabel}
          />
        )}
        {settings.plotType === 'bar' && (
          <Vis1DSettingsBar
            settings={settings}
            updateSettings={updateSettings}
            attributeOptions={attributeOptions}
            filterYLabel={filterYLabel}
            setFilterYLabel={setFilterYLabel}
            filterZLabel={filterZLabel}
            setFilterZLabel={setFilterZLabel}
          />
        )}

        {settings.plotType === 'histogram' && (
          <Vis1DSettingsHistogramCount
            settings={settings}
            updateSettings={updateSettings}
            attributeOptions={attributeOptions}
            filterYLabel={filterYLabel}
            setFilterYLabel={setFilterYLabel}
            filterZLabel={filterZLabel}
            setFilterZLabel={setFilterZLabel}
          />
        )}

        {/* }
        {(settings.plotType === 'scatter' ||
          settings.plotType === 'line' ||
          settings.xAxisLabel.name?.includes('Datum') ||
          settings.xAxisLabel.type === 'date' ||
          settings.xAxisLabel.type === 'datetime') && (
          <div className="mb-2">
            <DropdownSelector
              label="Y Axis"
              selectedValue={settings.yAxisLabel.name}
              options={attributeOptions}
              filter={filterYLabel}
              setFilter={setFilterYLabel}
              onChange={selected => {
                updateSettings({ yAxisLabel: { name: selected.name, type: selected.type } });
              }}
            />
          </div>
        )}
        {settings.plotType === 'histogram' && (
          <div className="mb-2">
            <Input
              type="dropdown"
              label="Aggregation:"
              value={settings.groupData}
              options={vis1DHistogramAggregationOptions}
              onChange={value => {
                updateSettings({ groupData: value as string });
              }}
            />
          </div>
        )}
        {(settings.plotType === 'bar' || settings.plotType === 'scatter' || settings.plotType === 'histogram') && (
          <div className="mb-2">
            <Input
              type="dropdown"
              label="Color:"
              value={settings.zAxisLabel}
              options={attributeOptions}
              onChange={value => {
                updateSettings({ zAxisLabel: value as string });
              }}
            />
          </div>
        )}
        {settings.plotType === 'histogram' && (
          <div className="mb-2">
            <Input type="boolean" label="Normalize: " value={settings.stack} onChange={val => updateSettings({ stack: val })} />
          </div>
        )}
        {(settings.xAxisLabel.name?.includes('Datum') ||
          settings.xAxisLabel.type === 'date' ||
          settings.xAxisLabel.type === 'datetime') && (
          <div className="mb-2">
            <Input
              type="dropdown"
              label="Group Time:"
              value={settings.groupData}
              options={['', 'hourly', 'daily', 'weekly', 'monthly', 'quarterly', 'yearly']}
              onChange={value => {
                updateSettings({ groupData: value as string });
              }}
            />
          </div>
        )}
        */}
        <div className="mb-2">
          <Input type="boolean" label="Show axis" value={settings.showAxis} onChange={val => updateSettings({ showAxis: val })} />
        </div>
      </div>
    </SettingsContainer>
  );
};

const Vis1DRef = React.createRef<Vis1DVisHandle>();

export const Vis1DComponent: VISComponentType<Vis1DProps> = {
  component: React.forwardRef((props: VisualizationPropTypes<Vis1DProps>, ref) => <Vis1D {...props} ref={Vis1DRef} />),
  settingsComponent: Vis1DSettings,
  settings: defaultSettings,
  exportImage: () => {
    if (Vis1DRef.current) {
      Vis1DRef.current.exportImageInternal();
    } else {
      console.error('1Dvis reference is not set.');
    }
  },
};

export default Vis1DComponent;
