const getWeekNumber = (date: Date): string => {
  const startDate = new Date(date.getFullYear(), 0, 1); // Start of the year
  const days = Math.floor((date.getTime() - startDate.getTime()) / (24 * 60 * 60 * 1000));
  const weekNumber = Math.ceil((days + 1) / 7); // Week number is 1-based
  return `${date.getFullYear()}-W${weekNumber.toString().padStart(2, '0')}`; // Format as "YYYY-WXX"
};

export const groupByTime = (xAxisData: string[], groupBy: string, additionalVariableData?: (string | number)[]) => {
  //console.log('xAxisData ', xAxisData);
  //console.log('additionalVariableData ', additionalVariableData);

  const parseDate = (dateStr: string) => {
    // Remove nanoseconds part and use just the standard "YYYY-MM-DD HH:MM:SS" part
    const cleanedDateStr = dateStr.split('.')[0];
    return new Date(cleanedDateStr);
  };

  // Grouping logic
  const groupedData = xAxisData.reduce(
    (acc, dateStr, index) => {
      const date = parseDate(dateStr);
      let groupKey: string;

      if (groupBy === 'yearly') {
        groupKey = date.getFullYear().toString(); // Group by year (e.g., "2012")
      } else if (groupBy === 'quarterly') {
        const month = date.getMonth() + 1; // Adjust month for zero-indexed months
        const quarter = Math.floor((month - 1) / 3) + 1; // Calculate quarter (Q1-Q4)
        groupKey = `${date.getFullYear()}-Q${quarter}`;
      } else if (groupBy === 'monthly') {
        // Group by month, e.g., "2012-07"
        groupKey = `${date.getFullYear()}-${(date.getMonth() + 1).toString().padStart(2, '0')}`;
      } else if (groupBy === 'weekly') {
        // Group by week, e.g., "2012-W15"
        groupKey = getWeekNumber(date);
      } else if (groupBy === 'daily') {
        // Group by day, e.g., "2012-07-15"
        groupKey = `${date.getFullYear()}-${(date.getMonth() + 1).toString().padStart(2, '0')}-${date.getDate().toString().padStart(2, '0')}`;
      } else if (groupBy === 'hourly') {
        // Group by hour, e.g., "2012-07-15 14"
        groupKey = `${date.getFullYear()}-${(date.getMonth() + 1).toString().padStart(2, '0')}-${date.getDate().toString().padStart(2, '0')} ${date.getHours().toString().padStart(2, '0')}`;
      } else if (groupBy === 'minutely') {
        // Group by minute, e.g., "2012-07-15 14:30"
        groupKey = `${date.getFullYear()}-${(date.getMonth() + 1).toString().padStart(2, '0')}-${date.getDate().toString().padStart(2, '0')} ${date.getHours().toString().padStart(2, '0')}:${date.getMinutes().toString().padStart(2, '0')}`;
      } else {
        // Default case: group by year (or some other grouping logic)
        groupKey = date.getFullYear().toString();
      }

      //console.log('additionalVariableData ', additionalVariableData);
      // Initialize the group if it doesn't exist
      if (!acc[groupKey]) {
        acc[groupKey] = additionalVariableData
          ? typeof additionalVariableData[0] === 'number'
            ? 0 // Initialize sum for numbers
            : [] // Initialize array for strings
          : 0; // Initialize count for no additional data
      }

      // Aggregate additional variable if provided
      if (additionalVariableData) {
        if (typeof additionalVariableData[index] === 'number') {
          acc[groupKey] = (acc[groupKey] as number) + (additionalVariableData[index] as number);
        } else if (typeof additionalVariableData[index] === 'string') {
          acc[groupKey] = [...(acc[groupKey] as string[]), additionalVariableData[index] as string];
        }
      } else {
        // Increment the count if no additionalVariableData
        acc[groupKey] = (acc[groupKey] as number) + 1;
      }

      return acc;
    },
    {} as Record<string, number | string[]>,
  );

  // Sorting logic outside the reduce block
  const sortedArray = Object.entries(groupedData)
    .map(([key, value]) => ({
      x: key,
      y: value,
      dateObj: (() => {
        if (key.includes('Q')) {
          return new Date(parseInt(key.split('-')[0], 10), (parseInt(key.split('-')[1].replace('Q', ''), 10) - 1) * 3);
        } else if (key.includes('-W')) {
          const [year, week] = key.split('-W').map(Number);
          const jan1 = new Date(year, 0, 1);
          return new Date(year, 0, jan1.getDate() + (week - 1) * 7);
        } else {
          return new Date(key);
        }
      })(),
    }))
    .sort((a, b) => a.dateObj.getTime() - b.dateObj.getTime());

  const xValuesGrouped = sortedArray.map(({ x }) => x);
  const yValuesGrouped = sortedArray.map(({ y }) => y);

  return { xValuesGrouped, yValuesGrouped };
};
