import React, { useEffect, useRef, useState } from 'react';
import Plot from 'react-plotly.js';
import { DropdownSelector } from '@/lib/components/DropdownSelector/DropdownSelector';
import { Vis1DInputData } from './types';
import { preparePlotDataAndLayoutLine } from '../prepareData/preparePlotLine';
import { Input } from '@/lib/components/inputs';

type Vis1DLineProps = {
  xData: Vis1DInputData;
  yData: Vis1DInputData;
  groupDataByTime?: string;
  showAxis: boolean;
};

export const Vis1DLine: React.FC<Vis1DLineProps> = ({ xData, yData, groupDataByTime, showAxis }) => {
  // common to all vis
  const internalRef = useRef<HTMLDivElement>(null);
  const [divSize, setDivSize] = useState({ width: 0, height: 0 });

  useEffect(() => {
    const handleResize = () => {
      if (internalRef.current) {
        const { width, height } = internalRef.current.getBoundingClientRect();
        setDivSize({ width, height });
      }
    };

    handleResize();
    window.addEventListener('resize', handleResize);
    if (internalRef.current) {
      new ResizeObserver(handleResize).observe(internalRef.current);
    }

    return () => {
      window.removeEventListener('resize', handleResize);
    };
  }, []);

  // specific to this vis
  const { data, layout } = preparePlotDataAndLayoutLine(xData, yData, showAxis, false, groupDataByTime);

  return (
    <div className="h-full w-full flex items-center justify-center overflow-hidden relative" ref={internalRef}>
      <Plot
        data={data}
        config={{
          responsive: true,
          scrollZoom: false,
          displayModeBar: false,
          displaylogo: false,
        }}
        layout={{
          ...layout,
          width: divSize.width,
          height: divSize.height,
          dragmode: false,
        }}
      />
    </div>
  );
};

interface Vis1DSettingsLineProps {
  settings: any;
  updateSettings: (updatedSettings: any) => void;
  attributeOptions: { name: string; type: string }[];
  filterYLabel: string;
  setFilterYLabel: React.Dispatch<React.SetStateAction<string>>;
}

export const Vis1DSettingsLine = ({
  settings,
  updateSettings,
  attributeOptions,
  filterYLabel,
  setFilterYLabel,
}: Vis1DSettingsLineProps) => {
  return (
    <div>
      <div className="mb-2">
        <DropdownSelector
          label="Y Axis"
          selectedValue={settings.yAxisLabel.name}
          options={attributeOptions}
          filter={filterYLabel}
          setFilter={setFilterYLabel}
          onChange={selected => {
            updateSettings({ yAxisLabel: { name: selected.name, type: selected.type } });
          }}
        />
        <div className="mb-2">
          <Input type="boolean" label="Stack: " value={settings.stack} onChange={val => updateSettings({ stack: val })} />
        </div>
        {(settings.xAxisLabel.name?.includes('Datum') ||
          settings.xAxisLabel.type === 'date' ||
          settings.xAxisLabel.type === 'datetime') && (
          <div className="mb-2">
            <Input
              type="dropdown"
              label="Group Time:"
              value={settings.groupDataByTime}
              options={['', 'hourly', 'daily', 'weekly', 'monthly', 'quarterly', 'yearly']}
              onChange={value => {
                updateSettings({ groupDataByTime: value as string });
              }}
            />
          </div>
        )}
      </div>
    </div>
  );
};
